package ch.std.jumpstartreactive.rest;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.std.jumpstartreactive.beans.ReactiveCitiesBean;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
public class ReactiveCityController {
	Logger logger = LoggerFactory.getLogger(ReactiveCityController.class);

	@GetMapping(path = "/cities", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
	public Flux<String> getAllCityNames() {
		logger.info("ReactiveCityController.getAllCityNames");
		return ReactiveCitiesBean.getCities();
	}

	@GetMapping(path = "/citieslimitrate", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
	public Flux<String> getAllCityNamesLimitRate() {
		logger.info("ReactiveCityController.getAllCityNamesLimitRate");
		return ReactiveCitiesBean.getCities().limitRate(2);
	}

	@GetMapping(path = "/citieslist", produces = MediaType.APPLICATION_NDJSON_VALUE)
	public Flux<List<String>> getCityList() {
		return ReactiveCitiesBean.getCitiesList();
	}

	@GetMapping(path = "/citieslistlimitrate", produces = MediaType.APPLICATION_NDJSON_VALUE)
	public Flux<List<String>> getCityListLimitRate() {
		return ReactiveCitiesBean.getCitiesList().limitRate(2);
	}

	@GetMapping(path = "/citiesmap", produces = MediaType.APPLICATION_NDJSON_VALUE)
	public Flux<Map<String, Object>> getCityMap() {
		return ReactiveCitiesBean.getCities().map(city -> {
			Map<String, Object> map = new HashMap<>();
			map.put("city", city);
			return map;
		});
	}

	@GetMapping(path = "/citiesmaplimitrate", produces = MediaType.APPLICATION_NDJSON_VALUE)
	public Flux<Map<String, Object>> getCityMapLimitRate() {
		return ReactiveCitiesBean.getCities().map(city -> {
			Map<String, Object> map = new HashMap<>();
			map.put("city", city);
			return map;
		}).limitRate(2);
	}

	@GetMapping(path = "/citiesmapjson", produces = MediaType.APPLICATION_JSON_VALUE)
	public Flux<Map<String, Object>> getCityMapJson() {
		return ReactiveCitiesBean.getCities().map(city -> Collections.singletonMap("city", city));
	}

	@GetMapping(path = "/city", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
	public Mono<String> getCity(@RequestParam(value = "value") String value) {
		return ReactiveCitiesBean.find(value);
	}

	@GetMapping(path = "/citymap", produces = MediaType.APPLICATION_NDJSON_VALUE)
	public Mono<Map<String, Object>> getCityMap(@RequestParam(value = "value") String value) {
		return ReactiveCitiesBean.find(value).map(city -> Collections.singletonMap("city", city));
	}

	@GetMapping(path = "/citymapjson", produces = MediaType.APPLICATION_JSON_VALUE)
	public Mono<Map<String, Object>> getCityMapJson(@RequestParam(value = "value") String value) {
		return ReactiveCitiesBean.find(value).map(city -> Collections.singletonMap("city", city));
	}

}
